package cn.estsh.i3plus.core.api.iservice.busi;

import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.platform.bean.SessionUser;
import cn.estsh.i3plus.pojo.platform.bean.SysTask;
import cn.estsh.i3plus.pojo.platform.bean.SysTaskPlan;
import cn.estsh.i3plus.pojo.platform.bean.SysTaskCycle;
import cn.estsh.i3plus.pojo.platform.platbean.SysLogTaskTime;
import io.swagger.annotations.ApiOperation;

import java.util.List;

/**
 * @Description : 任务计划业务接口
 * @Reference   :
 * @Author      : yunhao
 * @Date        : 2018-12-26 22:08
 * @Modify      :
 **/
public interface ISysTaskPlanService {

    /**
     * 新增任务计划
     * @param sysTaskPlan 任务计划
     * @return
     */
    @ApiOperation(value = "新增任务计划",notes = "新增任务计划")
    SysTaskPlan insertSysTaskPlan(SysTaskPlan sysTaskPlan);

    /**
     * 修改任务状态
     * @param id        任务ID
     * @param status    任务状态
     * @param user      当前会话用户
     */
    @ApiOperation(value = "修改任务计划状态",notes = "修改任务计划状态")
    void updateSysTaskPlanStatus(Long id, int status, SessionUser user);

    /**
     * 根据根据id删除数据
     * @param id
     */
    @ApiOperation(value = "根据根据id删除数据")
    void deleteSysTaskPlanById(Long id);

    /**
     * 查询所有任务计划
     * @return
     */
    @ApiOperation(value = "查询所有任务计划")
    List<SysTaskPlan> listSysTaskPlan();

    /**
     * 根据id查询任务计划数据
     * @param id
     * @return
     */
    @ApiOperation(value = "根据id查询任务计划数据",notes = "根据id查询任务计划数据")
    SysTaskPlan getSysTaskPlanById(Long id);

    /**
     * 任务计划复杂查询，分页，排序
     * @param sysTaskPlan
     * @param pager
     * @return
     */
    @ApiOperation(value = "任务计划复杂查询，分页，排序",notes = "任务计划复杂查询，分页，排序")
    ListPager querySysTaskPlanByPager(SysTaskPlan sysTaskPlan, Pager pager);

    /**
     * 根据任务计划名称与组名称查询
     * @param name
     * @param groupName
     * @return
     */
    @ApiOperation(value = "查询任务计划",notes = "根据任务计划名称与组名称查询")
    SysTaskPlan getSysTaskPlanByNameAndGroup(String name, String groupName);

    /**
     * 添加任务计划执行日志
     * @param sysLogTaskTime
     */
    @ApiOperation(value = "添加任务计划执行日志",notes = "添加任务计划执行日志")
    void logInsertSysLogTaskTime(SysLogTaskTime sysLogTaskTime);

    /**
     * 修改任务计划信息，不会更新quartz数据库
     * @param sysTaskPlan
     */
    @ApiOperation(value = "修改任务计划信息",notes = "修改任务计划信息，不会更新quartz数据库")
    void updateSysTaskPlan(SysTaskPlan sysTaskPlan);
}
